{% extends 'appearance/base.html' %}

{% load i18n %}

{% load navigation_tags %}

{% block title %}{% include 'appearance/partials/form_title.html' with non_html_title=True %}{% endblock %}

{% block content %}
    {% if list_as_items %}
        {% include 'appearance/generic_list_items_subtemplate.html' %}
    {% else %}
        {% include 'appearance/generic_list_subtemplate.html' %}
    {% endif %}
{% endblock content %}

