DEFAULT_CELERY_BROKER_LOGIN_METHOD = 'AMQPLAIN'
DEFAULT_CELERY_BROKER_URL = 'memory://'
DEFAULT_CELERY_BROKER_USE_SSL = None
DEFAULT_CELERY_RESULT_BACKEND = None

TEST_CELERY_RESULT_KEY = '__mayan_test_result'
TEST_CELERY_RESULT_VALUE = 'test'

WORKER_DEFAULT_CONCURRENCY = 0
