{% load i18n %}

{% load smart_settings_tags %}

{# appearance_app_template_nocache #}

{% smart_settings_check_changed as settings_changed %}
{% if settings_changed %}
    <div class="alert alert-dismissible alert-warning">
        <button type="button" class="close" data-dismiss="alert">&times;</button>
        <p><strong>{% trans 'Warning' %}</strong> {% trans 'Settings updated, restart your installation and refresh your browser for changes to take effect.' %}</p>
    </div>
{% endif %}
