{% load i18n %}
<img alt="{% trans 'Workflow preview' %}" class="img-responsive" name="{{ widget.name }}" {% include 'django/forms/widgets/attrs.html' %} src="{{ widget.value.get_api_image_url }}" style="margin:auto;" />

