from mayan.apps.appearance.classes import Icon

icon_api = Icon(driver_name='fontawesome', symbol='plug')
icon_api_documentation = Icon(driver_name='fontawesome', symbol='book')
icon_api_documentation_redoc = Icon(driver_name='fontawesome', symbol='book')
